#ifndef CB_CRITICAL_SECTION
#define CB_CRITICAL_SECTION

#ifdef _WIN32
// This is Windows specific, need to change this for other platforms.
#include <windows.h>
#include <process.h>

class CriticalSection
{
private:
    CRITICAL_SECTION _criticalSection;

public:
    CriticalSection()
    {
        InitializeCriticalSection(&_criticalSection);
    }
    ~CriticalSection()
    {
        DeleteCriticalSection(&_criticalSection);
    }

    void Enter()
    { 
        EnterCriticalSection(&_criticalSection); 
    }
    void Leave()
    { 
        LeaveCriticalSection(&_criticalSection); 
    }
};
#else
// Bogus implementation of this class. We won't use multithreading so critical sections are useless.

class CriticalSection
{
private:
	// no members

public:
    CriticalSection()
    {
    }
    ~CriticalSection()
    {
    }

    void Enter()
    { 
    }
    void Leave()
    { 
    }
};

#endif

class CriticalSectionLock
{
private:
    CriticalSection* _criticalSection;

public:
    CriticalSectionLock(CriticalSection& criticalSection)
        : _criticalSection(&criticalSection) 
    { 
        _criticalSection->Enter();
    }
    ~CriticalSectionLock()
    {
        _criticalSection->Leave();
    }
};

#endif
